@@ -127,7 +127,7 @@ struct BackupContext
    * Timestamp of the order in @e existing_order_id. Used to
    * select the most recent unpaid offer.
    */
-  struct GNUNET_TIME_Absolute existing_order_timestamp;
+  struct GNUNET_TIME_Timestamp existing_order_timestamp;
 
   /**
    * Expected total upload size.
@@ -371,7 +371,7 @@ proposal_cb (void *cls,
  */
 static void
 ongoing_payment_cb (void *cls,
-                    struct GNUNET_TIME_Absolute timestamp,
+                    struct GNUNET_TIME_Timestamp timestamp,
                     const char *order_id,
                     const struct TALER_ClaimTokenP *token,
                     const struct TALER_Amount *amount)
@@ -383,7 +383,9 @@ ongoing_payment_cb (void *cls,
                              &SH_annual_fee))
     return; /* can't re-use, fees changed */
   if ( (NULL == bc->existing_order_id) ||
-       (bc->existing_order_timestamp.abs_value_us < timestamp.abs_value_us) )
+       (GNUNET_TIME_timestamp_cmp (bc->existing_order_timestamp,
+                                   <,
+                                   timestamp)) )
   {
     GNUNET_free (bc->existing_order_id);
     bc->existing_order_id = GNUNET_strdup (order_id);
@@ -398,27 +400,57 @@ ongoing_payment_cb (void *cls,
  * Callback to process a GET /check-payment request
  *
  * @param cls our `struct BackupContext`
- * @param hr HTTP response details
  * @param osr order status
  */
 static void
 check_payment_cb (void *cls,
-                  const struct TALER_MERCHANT_HttpResponse *hr,
                   const struct TALER_MERCHANT_OrderStatusResponse *osr)
 {
   struct BackupContext *bc = cls;
+  const struct TALER_MERCHANT_HttpResponse *hr = &osr->hr;
 
   /* refunds are not supported, verify */
   bc->omgh = NULL;
-  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-              "Payment status checked: %d\n",
-              osr->status);
   GNUNET_CONTAINER_DLL_remove (bc_head,
                                bc_tail,
                                bc);
   MHD_resume_connection (bc->con);
   SH_trigger_daemon ();
-  switch (osr->status)
+  switch (hr->http_status)
+  {
+  case 0:
+    /* Likely timeout, complain! */
+    bc->response_code = MHD_HTTP_GATEWAY_TIMEOUT;
+    bc->resp = TALER_MHD_make_error (
+      TALER_EC_SYNC_GENERIC_BACKEND_TIMEOUT,
+      NULL);
+    return;
+  case MHD_HTTP_OK:
+    break; /* handled below */
+  default:
+    /* Unexpected backend response */
+    bc->response_code = MHD_HTTP_BAD_GATEWAY;
+    bc->resp = TALER_MHD_MAKE_JSON_PACK (
+      GNUNET_JSON_pack_uint64 ("code",
+                               TALER_EC_SYNC_GENERIC_BACKEND_ERROR),
+      GNUNET_JSON_pack_string ("hint",
+                               TALER_ErrorCode_get_hint (
+                                 TALER_EC_SYNC_GENERIC_BACKEND_ERROR)),
+      GNUNET_JSON_pack_uint64 ("backend-ec",
+                               (json_int_t) hr->ec),
+      GNUNET_JSON_pack_uint64 ("backend-http-status",
+                               (json_int_t) hr->http_status),
+      GNUNET_JSON_pack_allow_null (
+        GNUNET_JSON_pack_object_incref ("backend-reply",
+                                        (json_t *) hr->reply)));
+    return;
+  }
+
+  GNUNET_assert (MHD_HTTP_OK == hr->http_status);
+  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+              "Payment status checked: %d\n",
+              osr->details.success.status);
+  switch (osr->details.success.status)
   {
   case TALER_MERCHANT_OSC_PAID:
     {
